/*
Author:       Pascal Peter
Copyright:    (C) 2019 Pascal Peter
License:      GNU General Public License version 3
Website:      http://pascal.peter.free.fr
Email:        pascal.peter at free.fr
*/





let width = document.getElementById('d3-content').clientWidth, 
    height = 500,

    data = [5, 1, 3, 2, 0],
    radius = 100,
    animate = 0,
    speed = 4,

    last = Date.now(),
    angle = 0;

const svg = d3.select('#d3-content').append('svg')
    .attr('width', width)
    .attr('height', height)
    .append('g')
    .attr('transform', 'translate(' + width / 2 + ',' + height / 2 + ')scale(.5)')
    .append('g');

const frame = svg.append('g')
    .datum({radius: Infinity});





function draw() {
    // récupération des paramètres :
    data[0] = parseInt(document.getElementById('data-0').value);
    data[1] = parseInt(document.getElementById('data-1').value);
    data[2] = parseInt(document.getElementById('data-2').value);
    data[3] = parseInt(document.getElementById('data-3').value);
    if (data[2] < 1)
        data[3] = 0;
    data[4] = parseInt(document.getElementById('data-4').value);
    if (data[3] < 1)
        data[4] = 0;
    radius = parseInt(document.getElementById('radius').value);

    // on efface tout :
    frame.selectAll('*').remove();

    // on trace le premier engrenage :
    let offset = - width + data[1] * radius + 150;
    //console.log(offset);
    frame.append('g')
        .attr('class', 'turn-left')
        .attr('transform', 'translate(' + offset + ', 0)')
        .datum({teeth: data[0] * data[1], radius: radius * data[1], index: 1})
        .append('path')
        .attr('d', gear);

    // puis les autres :
    for (let i = 2; i < 5; i++) {
        offset += radius * (data[i - 1] + data[i]);
        let turn = 'turn-right';
        if ((i % 2) > 0)
            turn = 'turn-left';
        frame.append('g')
            .attr('class', turn)
            .attr('transform', 'translate(' + offset + ', 0)')
            .datum({teeth: data[0] * data[i], radius: radius * data[i], index: i})
            .append('path')
            .attr('d', gear);
    }

}





d3.selectAll('input[name=data]')
    .on('change', function() {
        //console.log(this.id);
        let i = parseInt(this.id.split('-')[1]);
        //console.log(i);
        data[i] = +this.value;
        //console.log(data);
        draw();
    });

d3.selectAll('input[name=radius]')
    .on('change', function() { radius = +this.value; draw(); });

d3.selectAll('input[name=speed]')
    .on('change', function() { speed = +this.value; });

d3.selectAll('input[name=check]')
    .on('change', function() {
        if (this.checked)
            animate = 1;
        else
            animate = 0;
        });





function gear(d) {
    if ((d.index % 2) > 0)
        d.radius = - d.radius;
    let r2 = Math.abs(d.radius),
        r0 = r2 - 16,
        r1 = r2 + 16,
        
        r3 = (d.radius > 0) ? (d.radius - 50) : (d.radius + 50),
        r4 = 16,
        
        da = Math.PI / d.teeth,
        p = 1;
    if (d.index == 2) {
        p = (data[0] * data[1]) % 2;
    }
    else if (d.index == 3) {
        p = (data[0] * (data[1] + data[2])) % 2;
        p = 1 - p;
    }
    else if (d.index == 4) {
        p = (data[0] * (data[1] + data[2] + data[3])) % 2;
    }
    let a0 = - Math.PI + da / 2 + da * (1 - p),
        i = -1,
        path = ['M', r0 * Math.cos(a0), ',', r0 * Math.sin(a0)];

    while (++i < d.teeth) path.push(
        'A', r0, ',', r0, ' 0 0,1 ', r0 * Math.cos(a0 += da), ',', r0 * Math.sin(a0),
        'L', r2 * Math.cos(a0), ',', r2 * Math.sin(a0),
        'L', r1 * Math.cos(a0 += da / 3), ',', r1 * Math.sin(a0),
        'A', r1, ',', r1, ' 0 0,1 ', r1 * Math.cos(a0 += da / 3), ',', r1 * Math.sin(a0),
        'L', r2 * Math.cos(a0 += da / 3), ',', r2 * Math.sin(a0),
        'L', r0 * Math.cos(a0), ',', r0 * Math.sin(a0));

    a0 = -Math.PI + da * (1 - p);
    let square = [
        'M', r0 * Math.cos(a0), ',', r0 * Math.sin(a0) - r4, 
        'l', r4, ',', r4, 
        'l', -r4, ',', r4, 
        'l', -r4, ',', -r4, 'Z'];
    path.push(square.join(''));

    path.push('M0,', -r3, 'A', r3, ',', r3, ' 0 0,0 0,', r3, 'A', r3, ',', r3, ' 0 0,0 0,', -r3, 'Z');
    return path.join('');
}

d3.timer(function() {
    if (animate > 0) {
        let now = Date.now(),
            delay = now - last;
        last = now;
        angle += delay * speed;
        let transform = function(d) { return 'rotate(' + angle / d.radius + ')'; };
        frame.selectAll('path').attr('transform', transform);
        frame.attr('transform', transform); // frame of reference
    }
    else
        last = Date.now();
});




        



function reset() {
    document.getElementById('animate').checked = false;
    document.getElementById('speed').value = speed;
    animate = 0;
    last = Date.now();
    angle = 0;
    draw();
}


window.onload=reset();
document.getElementById('reset').addEventListener('click', reset, false);

